
function OnFinish(selProj, selObj)
{
	try
	{
		var strProjectPath = wizard.FindSymbol('PROJECT_PATH');
		var strProjectName = wizard.FindSymbol('PROJECT_NAME');

		selProj = CreateCustomProject(strProjectName, strProjectPath);
		AddConfig(selProj, strProjectName);
		AddFilters(selProj);

		var InfFile = CreateCustomInfFile();
		AddFilesToCustomProj(selProj, strProjectName, strProjectPath, InfFile);
		PchSettings(selProj);
		InfFile.Delete();

		selProj.Object.Save();
	}
	catch(e)
	{
		if (e.description.length != 0)
			SetErrorInfo(e);
		return e.number
	}
}

function CreateCustomProject(strProjectName, strProjectPath)
{
	try
	{
		var strProjTemplatePath = wizard.FindSymbol('PROJECT_TEMPLATE_PATH');
		var strProjTemplate = '';
		strProjTemplate = strProjTemplatePath + '\\default.vcproj';

		var Solution = dte.Solution;
		var strSolutionName = "";
		if (wizard.FindSymbol("CLOSE_SOLUTION"))
		{
			Solution.Close();
			strSolutionName = wizard.FindSymbol("VS_SOLUTION_NAME");
			if (strSolutionName.length)
			{
				var strSolutionPath = strProjectPath.substr(0, strProjectPath.length - strProjectName.length);
				Solution.Create(strSolutionPath, strSolutionName);
			}
		}

		var strProjectNameWithExt = '';
		strProjectNameWithExt = strProjectName + '.vcproj';

		var oTarget = wizard.FindSymbol("TARGET");
		var prj;
		if (wizard.FindSymbol("WIZARD_TYPE") == vsWizardAddSubProject)  // vsWizardAddSubProject
		{
			var prjItem = oTarget.AddFromTemplate(strProjTemplate, strProjectNameWithExt);
			prj = prjItem.SubProject;
		}
		else
		{
			prj = oTarget.AddFromTemplate(strProjTemplate, strProjectPath, strProjectNameWithExt);
		}
		return prj;
	}
	catch(e)
	{
		throw e;
	}
}

function AddFilters(proj)
{
	try
	{
		// Add the folders to your project
		var strSrcFilter = wizard.FindSymbol('SOURCE_FILTER');
		var group = proj.Object.AddFilter('Source Files');
		group.Filter = strSrcFilter;
	}
	catch(e)
	{
		throw e;
	}
}

function AddConfig(proj, strProjectName)
{
	try
	{   
	    var ProjType = wizard.FindSymbol('LST_PROJECT');
	    var bUseUnicode = wizard.FindSymbol('RB_UNICODE');
	    
	    //
		// Debug (x86)
		//
	    
		var config = proj.Object.Configurations('Debug');
		
		config.IntermediateDirectory = '$(ConfigurationName)';
		config.OutputDirectory = '$(ConfigurationName)';
		
		if (ProjType != 'Win32Dll')
		    config.ConfigurationType = 1; // exe
		else
		    config.ConfigurationType = 2; // dll
				
		if (bUseUnicode == true)
		    config.CharacterSet = 1; // unicode
		else
		    config.CharacterSet = 0; // ascii

		var CLTool = config.Tools('VCCLCompilerTool');
		
		CLTool.Optimization = 0;
		CLTool.MinimalRebuild = true;
		CLTool.BasicRuntimeChecks = 3;
		CLTool.RuntimeLibrary = 0;
		CLTool.UsePrecompiledHeader = 0;
		CLTool.WarningLevel = 3;
		CLTool.Detect64BitPortabilityProblems = true;
		CLTool.DebugInformationFormat = 4;
				
		if (ProjType == 'Win32Exe')
		    CLTool.PreprocessorDefinitions = 'WIN32;_DEBUG;_WINDOWS';
		else if (ProjType == 'Console')
		    CLTool.PreprocessorDefinitions = 'WIN32;_DEBUG;_CONSOLE';
		 else 
		    CLTool.PreprocessorDefinitions = 'WIN32;_DEBUG;_WINDOWS;_USRDLL';

		var LinkTool = config.Tools('VCLinkerTool');
		
		LinkTool.LinkIncremental = 2;
		LinkTool.GenerateDebugInformation = true;
		
		if (ProjType != 'Console')
		    LinkTool.SubSystem = 2; // win32
		else
		    LinkTool.SubSystem = 1; // console
		    
		LinkTool.TargetMachine = 1;
		
		//
		// Release (x86)
		//

		config = proj.Object.Configurations('Release');
		
		config.IntermediateDirectory = '$(ConfigurationName)';
		config.OutputDirectory = '$(ConfigurationName)';
		
		if (ProjType != 'Win32Dll')
		    config.ConfigurationType = 1; // exe
		else
		    config.ConfigurationType = 2; // dll
		
		if (bUseUnicode == true)
		    config.CharacterSet = 1; // unicode
		else
		    config.CharacterSet = 0; // ascii
		    
		var CLTool = config.Tools('VCCLCompilerTool');
		
		CLTool.Optimization = 1;
		CLTool.FavorSizeOrSpeed = 2;
		CLTool.WholeProgramOptimization = false;
		CLTool.RuntimeLibrary = 0;
		CLTool.StructMemberAlignment = 1;
		CLTool.BufferSecurityCheck = false;
		CLTool.UsePrecompiledHeader = 0;
		CLTool.WarningLevel = 3;
		CLTool.Detect64BitPortabilityProblems = true;
		CLTool.DebugInformationFormat = 3;
		
		if (ProjType == 'Win32Exe')
		    CLTool.PreprocessorDefinitions = 'WIN32;NDEBUG;_WINDOWS';
		else if (ProjType == 'Console')
		    CLTool.PreprocessorDefinitions = 'WIN32;NDEBUG;_CONSOLE';
		 else 
		    CLTool.PreprocessorDefinitions = 'WIN32;NDEBUG;_WINDOWS;_USRDLL';
		    
		var LinkTool = config.Tools('VCLinkerTool');
		
		LinkTool.AdditionalDependencies = 'small_libc_x86.lib';
		LinkTool.LinkIncremental = 1;
		LinkTool.IgnoreAllDefaultLibraries = true;
		LinkTool.GenerateDebugInformation = false;
		
		if (ProjType != 'Console')
		    LinkTool.SubSystem = 2; // win32
		else
		    LinkTool.SubSystem = 1; // console
		
		LinkTool.OptimizeReferences = 2;
		LinkTool.EnableCOMDATFolding = 2;
		LinkTool.TargetMachine = 1;
	}
	catch(e)
	{
		throw e;
	}
}

function PchSettings(proj)
{
	// TODO: specify pch settings
}

function DelFile(fso, strWizTempFile)
{
	try
	{
		if (fso.FileExists(strWizTempFile))
		{
			var tmpFile = fso.GetFile(strWizTempFile);
			tmpFile.Delete();
		}
	}
	catch(e)
	{
		throw e;
	}
}

function CreateCustomInfFile()
{
	try
	{
		var fso, TemplatesFolder, TemplateFiles, strTemplate;
		fso = new ActiveXObject('Scripting.FileSystemObject');

		var TemporaryFolder = 2;
		var tfolder = fso.GetSpecialFolder(TemporaryFolder);
		var strTempFolder = tfolder.Drive + '\\' + tfolder.Name;

		var strWizTempFile = strTempFolder + "\\" + fso.GetTempName();

		var strTemplatePath = wizard.FindSymbol('TEMPLATES_PATH');
		var strInfFile = strTemplatePath + '\\Templates.inf';
		wizard.RenderTemplate(strInfFile, strWizTempFile);

		var WizTempFile = fso.GetFile(strWizTempFile);
		return WizTempFile;
	}
	catch(e)
	{
		throw e;
	}
}

function GetTargetName(strName, strProjectName)
{
	try
	{
		// TODO: set the name of the rendered file based on the template filename
		var strTarget = strName;

		if (strName == 'readme.txt')
			strTarget = 'ReadMe.txt';
			
		if (strName == 'main.h')
		    strTarget = strProjectName + '.h';
		    
		if (strName == 'main.cpp')
		    strTarget = strProjectName + '.cpp';
		    
		if (strName == 'crt.cpp')
		    strTarget = strProjectName + ' CRT.cpp';
		    
		return strTarget; 
	}
	catch(e)
	{
		throw e;
	}
}

function AddFilesToCustomProj(proj, strProjectName, strProjectPath, InfFile)
{
	try
	{
		var projItems = proj.ProjectItems

		var strTemplatePath = wizard.FindSymbol('TEMPLATES_PATH');

		var strTpl = '';
		var strName = '';

		var strTextStream = InfFile.OpenAsTextStream(1, -2);
		while (!strTextStream.AtEndOfStream)
		{
			strTpl = strTextStream.ReadLine();
			if (strTpl != '')
			{
				strName = strTpl;
				var strTarget = GetTargetName(strName, strProjectName);
				var strTemplate = strTemplatePath + '\\' + strTpl;
				var strFile = strProjectPath + '\\' + strTarget;

				var bCopyOnly = false;  //"true" will only copy the file from strTemplate to strTarget without rendering/adding to the project
				var strExt = strName.substr(strName.lastIndexOf("."));
				if(strExt==".bmp" || strExt==".ico" || strExt==".gif" || strExt==".rtf" || strExt==".css" || strExt==".lib")
					bCopyOnly = true;
				wizard.RenderTemplate(strTemplate, strFile, bCopyOnly);
				
				if (strExt != ".lib")
				    proj.Object.AddFile(strFile);
			}
		}
		strTextStream.Close();
	}
	catch(e)
	{
		throw e;
	}
}
