#include <errno.h>
#include <stdarg.h>

#ifndef NULL
#define NULL 0
#endif

#ifndef ULONG_MAX
#define ULONG_MAX 0xffffffffUL  
#endif

#ifndef LONG_MIN
#define LONG_MIN (-2147483647L - 1)  
#define LONG_MAX 2147483647L
#endif

#ifndef STD_INPUT_HANDLE
#define STD_INPUT_HANDLE    ((unsigned long)-10)
#define STD_OUTPUT_HANDLE   ((unsigned long)-11)
#endif

//
// Define AVOID_IF_POSSIBLE_WINAPI if you don't want
// to use windows APIs when possible
//
//#define AVOID_IF_POSSIBLE_WINAPI

//
// Define external functions (apis)
//

extern "C" int _stdcall lstrlenA(const char *lpString);
extern "C" int _stdcall lstrlenW(const wchar_t *lpString);

extern "C" char * _stdcall lstrcpyA(char *lpString1, const char *lpString2);
extern "C" wchar_t * _stdcall lstrcpyW(wchar_t *lpString1, const wchar_t *lpString2);

extern "C" char * _stdcall lstrcpynA(char *lpString1, const char *lpString2, int iMaxLength);
extern "C" wchar_t * _stdcall lstrcpynW(wchar_t *lpString1, const wchar_t *lpString2, int iMaxLength);

extern "C" char * _stdcall lstrcatA(char *lpString1, const char *lpString2);
extern "C" wchar_t * _stdcall lstrcatW(wchar_t *lpString1, const wchar_t *lpString2);

extern "C" int _stdcall lstrcmpA(const char *lpString1, const char *lpString2);
extern "C" int _stdcall lstrcmpW(const wchar_t *lpString1, const wchar_t *lpString2);

extern "C" int _stdcall lstrcmpiA(const char *lpString1, const char *lpString2);
extern "C" int _stdcall lstrcmpiW(const wchar_t *lpString1, const wchar_t *lpString2);

extern "C" unsigned long _stdcall CharUpperBuffA(char *lpsz, unsigned long cchLength);
extern "C" unsigned long _stdcall CharLowerBuffA(char *lpsz, unsigned long cchLength);
extern "C" unsigned long _stdcall CharUpperBuffW(wchar_t *lpsz, unsigned long cchLength);
extern "C" unsigned long _stdcall CharLowerBuffW(wchar_t *lpsz, unsigned long cchLength);

extern "C" void * _stdcall GetProcessHeap(void);
extern "C" void * _stdcall HeapAlloc(void *hHeap,  unsigned long dwFlags, void *dwBytes);
extern "C" long _stdcall HeapFree(void *hHeap, unsigned long dwFlags, void *lpMem);

extern "C" void * _stdcall GetStdHandle(unsigned long nStdHandle);

extern "C" long _stdcall ReadConsoleA(void *hConsoleInput, void *lpBuffer, unsigned long nNumberOfCharsToRead,
									  unsigned long *lpNumberOfCharsRead, void *lpReserved);
extern "C" long _stdcall ReadConsoleW(void *hConsoleInput, void *lpBuffer, unsigned long nNumberOfCharsToRead,
									  unsigned long *lpNumberOfCharsRead, void *lpReserved);

//
// Memory Functions
//

extern "C" void * __cdecl memset(void *dst, int val, size_t count)
{
	void *start = dst;

	while (count--) 
	{
		*(char *) dst = (char) val;
		dst = (char *) dst + 1;
	}

	return start;
}

extern "C" void * __cdecl memcpy(void *dst, const void *src, size_t count)
{
	void * ret = dst;

	while (count--) 
	{
		*(char *) dst = *(char *) src;
		dst = (char *) dst + 1;
		src = (char *) src + 1;
	}

	return ret;
}

extern "C" void * __cdecl memmove(void *dst, const void *src, size_t count)
{
	void * ret = dst;

	if (dst <= src || (char *)dst >= ((char *) src + count)) 
	{
		while (count--) 
		{
			*(char *) dst = *(char *)src;
			dst = (char *) dst + 1;
			src = (char *) src + 1;
		}
	}
	else 
	{
		dst = (char *) dst + count - 1;
		src = (char *) src + count - 1;

		while (count--) 
		{
			*(char *) dst = *(char *) src;
			dst = (char *) dst - 1;
			src = (char *) src - 1;
		}
	}

	return ret;
}

extern "C" int __cdecl memcmp(const void *buf1, const void *buf2, size_t count)
{
	if (!count) return 0;

	while (--count && *(char *) buf1 == *(char *) buf2) 
	{
		buf1 = (char *) buf1 + 1;
		buf2 = (char *) buf2 + 1;
	}

	return (*((unsigned char *) buf1) - *((unsigned char *) buf2));
}

extern "C" const void * __cdecl memchr(const void * buf, int chr, size_t cnt)
{
	while (cnt && (*(unsigned char *) buf != (unsigned char) chr)) 
	{
		buf = (unsigned char *) buf + 1;
		cnt--;
	}

	return (cnt ? (void *) buf : NULL);
}

//
// Secure Memory Functions
//

extern "C" errno_t __cdecl memcpy_s(void *dst, size_t sizeInBytes, const void *src, size_t count)
{
	if (count == 0) return 0;

	if (src == NULL || sizeInBytes < count)
	{
		memset(dst, 0, sizeInBytes);

		return EINVAL;
	}

	memcpy(dst, src, count);

	return 0;
}

//
//  Ansi String Functions
//

extern "C" size_t __cdecl strlen(const char *str)
{

#ifndef AVOID_IF_POSSIBLE_WINAPI

	return (size_t) lstrlenA(str);

#else

	const char *eos = str;

	while (*eos++) ;

	return (eos - str - 1);

#endif
}

extern "C" char * __cdecl strcpy(char *dst, const char *src)
{

#ifndef AVOID_IF_POSSIBLE_WINAPI

	return lstrcpyA(dst, src);

#else

	char *cp = dst;

	while (*cp++ = *src++);

	return dst;

#endif
}

extern "C" char * __cdecl strncpy(char *dest, const char *source, size_t count)
{

#ifndef AVOID_IF_POSSIBLE_WINAPI

	return lstrcpynA(dest, source, (int) count);

#else

	char *start = dest;

	while (count && (*dest++ = *source++)) 
		count--;

	if (count)                              
		while (--count)
			*dest++ = '\0';

	return start;

#endif

}

extern "C" char * __cdecl strcat(char *dst, const char *src)
{

#ifndef AVOID_IF_POSSIBLE_WINAPI

	return lstrcatA(dst, src);

#else

	char *cp = dst;

	while (*cp) cp++;                  

	while (*cp++ = *src++);      

	return dst;      

#endif

}

extern "C" char * __cdecl strncat(char * front, const char * back, size_t count)
{
	char *start = front;

	while (*front++)
		;
	front--;

	while (count--)
		if (!(*front++ = *back++))
			return start;

	*front = '\0';

	return start;
}

extern "C" int __cdecl strcmp(const char *src, const char *dst)
{

#ifndef AVOID_IF_POSSIBLE_WINAPI

	return lstrcmpA(src, dst);

#else

	int ret = 0;

	while (!(ret = *(unsigned char *) src - *(unsigned char *) dst) && *dst)
		++src, ++dst;

	if (ret < 0) ret = -1;
	else if (ret > 0) ret = 1;

	return ret;

#endif

}

extern "C" int __cdecl strncmp(const char * src, const char * dst, size_t count)
{
	int ret = 0;

	while (count-- && !(ret = *(unsigned char *) src - *(unsigned char *) dst) && *dst)
		++src, ++dst;

	if (ret < 0) ret = -1;
	else if (ret > 0) ret = 1;

	return ret;
}

extern "C" int __cdecl _stricmp(const char *dst, const char *src)
{
	// Best method to reduce size: use windows api

	 return lstrcmpiA(dst, src);
}

extern "C" char * __cdecl strupr(char *Str)
{
	// Best method to reduce size: use windows api

	CharUpperBuffA(Str, (unsigned long) strlen(Str));

	return Str;
}

extern "C" char * __cdecl strlwr(char *Str)
{
	// Best method to reduce size: use windows api

	CharLowerBuffA(Str, (unsigned long) strlen(Str));

	return Str;
}

extern "C" const char * __cdecl strchr(const char *string, int ch)
{
	while (*string && *string != (char) ch)
		string++;

	if (*string == (char) ch)
		return (char *) string;

	return NULL;
}

extern "C" const char * __cdecl strstr(const char *str1, const char *str2)
{
	char *cp = (char *) str1;
	char *s1, *s2;

	if (!*str2)
		return (char *) str1;

	while (*cp)
	{
		s1 = cp;
		s2 = (char *) str2;

		while (*s1 && *s2 && !(*s1 - *s2))
			s1++, s2++;

		if (!*s2) return cp;

		cp++;
	}

	return NULL;
}

extern "C" long __cdecl strtol(const char *nptr, char **endptr, int base)
{
	const char *s = nptr;
	unsigned long acc;
	int c;
	unsigned long cutoff;
	int neg = 0, any, cutlim;

	do 
	{
		c = *s++;
	} while (c == '0x20'); // is space

	if (c == '-') 
	{
		neg = 1;
		c = *s++;
	} else if (c == '+')
		c = *s++;
	if ((base == 0 || base == 16) &&
		c == '0' && (*s == 'x' || *s == 'X')) 
	{
			c = s[1];
			s += 2;
			base = 16;
	}
	if (base == 0)
		base = c == '0' ? 8 : 10;

	cutoff = neg ? -(unsigned long) LONG_MIN : LONG_MAX;
	cutlim = cutoff % (unsigned long)base;
	cutoff /= (unsigned long)base;

	for (acc = 0, any = 0;; c = *s++) {
		if (c >= '0' && c <= '9') // is digit
			c -= '0';
		else if ((c >= 64 && c <= 90) || (c >= 97 && c <= 122)) // is alpha
			c -= (c >= 64 && c <= 90) /* is upper */ ? 'A' - 10 : 'a' - 10;
		else
			break;
		if (c >= base)
			break;
		if (any < 0 || acc > cutoff || acc == cutoff && c > cutlim)
			any = -1;
		else 
		{
			any = 1;
			acc *= base;
			acc += c;
		}
	}
	if (any < 0) 
	{
		acc = neg ? LONG_MIN : LONG_MAX;
	} else if (neg)
		acc = -acc;
	if (endptr != 0)
		*endptr = (char *) (any ? s - 1 : nptr);

	return acc;
}

extern "C" unsigned long __cdecl strtoul(const char *nptr, char **endptr, int base)
{
	const char *s = nptr;
	unsigned long acc;
	int c;
	unsigned long cutoff;
	int neg = 0, any, cutlim;

	do 
	{
		c = *s++;
	} while (c == '0x20'); // is space

	if (c == '-')
	{
		neg = 1;
		c = *s++;
	} else if (c == '+')
		c = *s++;

	if ((base == 0 || base == 16) && c == '0' && (*s == 'x' || *s == 'X')) 
	{
			c = s[1];
			s += 2;
			base = 16;
	}

	if (base == 0)
		base = c == '0' ? 8 : 10;

	cutoff = (unsigned long) ULONG_MAX / (unsigned long) base; 
	cutlim = (unsigned long) ULONG_MAX % (unsigned long) base;

	for (acc = 0, any = 0; ; c = *s++) 
	{
		if (c >= '0' && c <= '9') // is digit
			c -= '0';
		else if ((c >= 64 && c <= 90) || (c >= 97 && c <= 122)) // is alpha
			c -= (c >= 64 && c <= 90) /* is upper */ ? 'A' - 10 : 'a' - 10;
		else
			break;
		if (c >= base)
			break;
		if (any < 0 || acc > cutoff || (acc == cutoff && c > cutlim))
			any = -1;
		else 
		{
			any = 1;
			acc *= base;
			acc += c;
		}
	}

	if (any < 0) 
	{
		acc = ULONG_MAX;
	} else if (neg)
		acc = (unsigned) -acc;
	if (endptr != 0)
		*endptr = (char *) (any ? s - 1 : nptr);

	return acc;
}

extern "C" void _splitpath(const char *path, char *drv, char *dir, char *name, char *ext)
{
	const char *end, *p, *s;

	if (path[0] && path[1] == ':') 
	{
		if (drv) 
		{
			*drv++ = *path++;
			*drv++ = *path++;
			*drv = '\0';
		}
	} else if (drv)
		*drv = '\0';

	for (end = path; *end && *end != ':';)
		end++;

	for (p = end; p > path && *--p != '\\' && *p != '/'; )
		if (*p == '.') 
		{
			end = p;
			break;
		}

		if (ext)
			for (s = end; (*ext = *s++); )
				ext++;

		for(p = end; p > path; )
			if (*--p == '\\' || *p == '/') 
			{
				p++;
				break;
			}

			if (name) 
			{
				for(s = p; s < end; )
					*name++ = *s++;

				*name = '\0';
			}

			if (dir) 
			{
				for(s = path; s < p; )
					*dir++ = *s++;

				*dir = '\0';
			}
}

//
//  Ansi Secure String Functions
//

extern "C" errno_t __cdecl strcpy_s(char *Dst, size_t DstSize, const char *Src)
{

#ifndef AVOID_IF_POSSIBLE_WINAPI

	lstrcpynA(Dst, Src, (int) DstSize);

#else

	strncpy(Dst, Src, DstSize);

#endif

	return 0;
}

extern "C" errno_t __cdecl strncpy_s(char *Dst, size_t DstSize, const char *Src, size_t MaxCount)
{
	size_t size = (DstSize - 1) < MaxCount ? (DstSize - 1) : MaxCount;

#ifndef AVOID_IF_POSSIBLE_WINAPI

	lstrcpynA(Dst, Src, (int) size);

#else

	strncpy(Dst, Src, size);

#endif

	return 0;
}

extern "C" errno_t __cdecl strcat_s(char *Dst, size_t DstSize, const char *Src)
{
	size_t DstLen = strlen(Dst), SrcLen = strlen(Src);
	size_t Left = (DstSize - 1) - DstLen;
	size_t size = Left < SrcLen ? Left : SrcLen;

	strncat(Dst, Src, size);

	return 0;
}

extern "C" errno_t __cdecl strncat_s(char *Dst, size_t DstSize, const char *Src, size_t MaxCount)
{
	size_t DstLen = strlen(Dst);
	size_t Left = (DstSize - 1) - DstLen;
	size_t size = Left < MaxCount ? Left : MaxCount;

	strncat(Dst, Src, size);

	return 0;
}

extern "C" errno_t  __cdecl _splitpath_s(const char *FullPath, char *Drive, size_t DriveSize, 
										 char *Dir, size_t DirSize, char *Filename, size_t FilenameSize, 
										 char *Ext, size_t ExtSize)
{
	// Fake secure

	_splitpath(FullPath, Drive, Dir, Filename, Ext);

	return 0;
}


//
//  Unicode String Functions
//

extern "C" size_t __cdecl wcslen(const wchar_t *wcs)
{

#ifndef AVOID_IF_POSSIBLE_WINAPI

	return (size_t) lstrlenW(wcs);

#else

	const wchar_t *eos = wcs;

	while (*eos++) ;

	return (size_t) (eos - wcs - 1);

#endif
}

extern "C" wchar_t * __cdecl wcscpy(wchar_t *dst, const wchar_t *src)
{

#ifndef AVOID_IF_POSSIBLE_WINAPI

	return lstrcpyW(dst, src);

#else

	wchar_t *cp = dst;

	while (*cp++ = *src++) ;             

	return dst;

#endif
}

extern "C" wchar_t * __cdecl wcsncpy(wchar_t *dest, const wchar_t *source, size_t count)
{

#ifndef AVOID_IF_POSSIBLE_WINAPI

	return lstrcpynW(dest, source, (int) count);

#else

	wchar_t *start = dest;

	while (count && (*dest++ = *source++))   
		count--;

	if (count)                    
		while (--count)
			*dest++ = L'\0';

	return start;

#endif
}

extern "C" wchar_t * __cdecl wcscat(wchar_t *dst, const wchar_t *src)
{

#ifndef AVOID_IF_POSSIBLE_WINAPI

	return lstrcatW(dst, src);

#else

	wchar_t *cp = dst;

	while (*cp) cp++;                  

	while (*cp++ = *src++) ;       

	return dst;      

#endif

}

extern "C" wchar_t * __cdecl wcsncat(wchar_t *front, const wchar_t *back, size_t count)
{
	wchar_t *start = front;

	while (*front++) ;

	front--;

	while (count--)
		if (!(*front++ = *back++))
			return start;

	*front = L'\0';

	return start ;
}

extern "C" int __cdecl wcscmp(const wchar_t *src, const wchar_t *dst)
{

#ifndef AVOID_IF_POSSIBLE_WINAPI

	return lstrcmpW(src, dst);

#else

	int ret = 0;

	while (!(ret = (int)(*src - *dst)) && *dst)
		++src, ++dst;

	if (ret < 0)
		ret = -1;
	else if (ret > 0)
		ret = 1;

	return ret; 

#endif

}

extern "C" int __cdecl wcsncmp(const wchar_t *src, const wchar_t *dst, size_t count)
{
	int ret = 0;

	while (count-- && !(ret = (int)(*src - *dst)) && *dst)
		++src, ++dst;

	if (ret < 0)
		ret = -1;
	else if (ret > 0)
		ret = 1;

	return ret; 
}

extern "C" int __cdecl _wcsicmp(const wchar_t *dst, const wchar_t *src)
{
	// Best method to reduce size: use windows api

	 return lstrcmpiW(dst, src);
}

extern "C" wchar_t * __cdecl wcsupr(wchar_t *Str)
{
	// Best method to reduce size: use windows api

	CharUpperBuffW(Str, (unsigned long) wcslen(Str));

	return Str;
}

extern "C" wchar_t * __cdecl wcslwr(wchar_t *Str)
{
	// Best method to reduce size: use windows api

	CharLowerBuffW(Str, (unsigned long) wcslen(Str));

	return Str;
}

extern "C" const wchar_t * __cdecl wcschr(const wchar_t *string, wchar_t ch)
{
	while (*string && *string != (wchar_t) ch)
		string++;

	if (*string == (wchar_t) ch)
		return (wchar_t *) string;

	return NULL;
}

extern "C" const wchar_t * __cdecl wcsstr(const wchar_t *wcs1, const wchar_t *wcs2)
{
	wchar_t *cp = (wchar_t *) wcs1;
	wchar_t *s1, *s2;

	if (!*wcs2)
		return (wchar_t *) wcs1;

	while (*cp)
	{
		s1 = cp;
		s2 = (wchar_t *) wcs2;

		while (*s1 && *s2 && !(*s1-*s2))
			s1++, s2++;

		if (!*s2) return cp;

		cp++;
	}

	return NULL;
}

extern "C" long __cdecl wcstol(const wchar_t *nptr, wchar_t **endptr, int base)
{
	const wchar_t *s = nptr;
	unsigned long acc;
	int c;
	unsigned long cutoff;
	int neg = 0, any, cutlim;

	do 
	{
		c = *s++;
	} while (c == '0x20'); // is space

	if (c == '-') 
	{
		neg = 1;
		c = *s++;
	} else if (c == '+')
		c = *s++;
	if ((base == 0 || base == 16) &&
		c == '0' && (*s == 'x' || *s == 'X')) 
	{
		c = s[1];
		s += 2;
		base = 16;
	}
	if (base == 0)
		base = c == '0' ? 8 : 10;

	cutoff = neg ? -(unsigned long) LONG_MIN : LONG_MAX;
	cutlim = cutoff % (unsigned long)base;
	cutoff /= (unsigned long)base;

	for (acc = 0, any = 0;; c = *s++) {
		if (c >= '0' && c <= '9') // is digit
			c -= '0';
		else if ((c >= 64 && c <= 90) || (c >= 97 && c <= 122)) // is alpha
			c -= (c >= 64 && c <= 90) /* is upper */ ? 'A' - 10 : 'a' - 10;
		else
			break;
		if (c >= base)
			break;
		if (any < 0 || acc > cutoff || acc == cutoff && c > cutlim)
			any = -1;
		else 
		{
			any = 1;
			acc *= base;
			acc += c;
		}
	}
	if (any < 0) 
	{
		acc = neg ? LONG_MIN : LONG_MAX;
	} else if (neg)
		acc = -acc;
	if (endptr != 0)
		*endptr = (wchar_t *) (any ? s - 1 : nptr);

	return acc;
}

extern "C" unsigned long __cdecl wcstoul(const wchar_t *nptr, wchar_t **endptr, int base)
{
	const wchar_t *s = nptr;
	unsigned long acc;
	int c;
	unsigned long cutoff;
	int neg = 0, any, cutlim;

	do 
	{
		c = *s++;
	} while (c == '0x20'); // is space

	if (c == '-')
	{
		neg = 1;
		c = *s++;
	} else if (c == '+')
		c = *s++;

	if ((base == 0 || base == 16) && c == '0' && (*s == 'x' || *s == 'X')) 
	{
		c = s[1];
		s += 2;
		base = 16;
	}

	if (base == 0)
		base = c == '0' ? 8 : 10;

	cutoff = (unsigned long) ULONG_MAX / (unsigned long) base; 
	cutlim = (unsigned long) ULONG_MAX % (unsigned long) base;

	for (acc = 0, any = 0; ; c = *s++) 
	{
		if (c >= '0' && c <= '9') // is digit
			c -= '0';
		else if ((c >= 64 && c <= 90) || (c >= 97 && c <= 122)) // is alpha
			c -= (c >= 64 && c <= 90) /* is upper */ ? 'A' - 10 : 'a' - 10;
		else
			break;
		if (c >= base)
			break;
		if (any < 0 || acc > cutoff || (acc == cutoff && c > cutlim))
			any = -1;
		else 
		{
			any = 1;
			acc *= base;
			acc += c;
		}
	}

	if (any < 0) 
	{
		acc = ULONG_MAX;
	} else if (neg)
		acc = (unsigned) -acc;
	if (endptr != 0)
		*endptr = (wchar_t *) (any ? s - 1 : nptr);

	return acc;
}

extern "C" void _wsplitpath(const wchar_t *path, wchar_t *drv, wchar_t *dir, wchar_t *name, wchar_t *ext)
{
	const wchar_t *end, *p, *s;  

	if (path[0] && path[1] == ':') 
	{
		if (drv) 
		{
			*drv++ = *path++;
			*drv++ = *path++;
			*drv = L'\0';
		}
	} else if (drv)
		*drv = L'\0';

	for (end = path; *end && *end != L':'; )
		end++;

	for (p = end; p > path && *--p != L'\\' && *p != L'/'; )
		if (*p == L'.') 
		{
			end = p;
			break;
		}

		if (ext)
			for (s = end; (*ext = *s++); )
				ext++;

		for (p = end; p > path; )
			if (*--p == '\\' || *p == '/') 
			{
				p++;
				break;
			}

			if (name) 
			{
				for (s = p; s < end; )
					*name++ = *s++;

				*name = L'\0';
			}

			if (dir) 
			{
				for(s = path; s < p; )
					*dir++ = *s++;

				*dir = L'\0';
			}
}

//
//  Unicode Secure String Functions
//

extern "C" errno_t __cdecl wcscpy_s(wchar_t *Dst, size_t DstSize, const wchar_t *Src)
{

#ifndef AVOID_IF_POSSIBLE_WINAPI

	lstrcpynW(Dst, Src, (int) DstSize);

#else

	wcsncpy(Dst, Src, DstSize);

#endif

	return 0;
}

extern "C" errno_t __cdecl wcsncpy_s(wchar_t *Dst, size_t DstSize, const wchar_t *Src, size_t MaxCount)
{
	size_t size = (DstSize - 1) < MaxCount ? (DstSize - 1) : MaxCount;

#ifndef AVOID_IF_POSSIBLE_WINAPI

	lstrcpynW(Dst, Src, (int) size);

#else

	wcsncpy(Dst, Src, size);

#endif

	return 0;
}

extern "C" errno_t __cdecl wcscat_s(wchar_t *Dst, size_t DstSize, const wchar_t *Src)
{
	size_t DstLen = wcslen(Dst), SrcLen = wcslen(Src);
	size_t Left = (DstSize - 1) - DstLen;
	size_t size = Left < SrcLen ? Left : SrcLen;

	wcsncat(Dst, Src, size);

	return 0;
}

extern "C" errno_t __cdecl wcsncat_s(wchar_t *Dst, size_t DstSize, const wchar_t *Src, size_t MaxCount)
{
	size_t DstLen = wcslen(Dst);
	size_t Left = (DstSize - 1) - DstLen;
	size_t size = Left < MaxCount ? Left : MaxCount;

	wcsncat(Dst, Src, size);

	return 0;
}

extern "C" errno_t __cdecl _wsplitpath_s(const wchar_t *FullPath, wchar_t *Drive, size_t DriveSizeInWords, 
										 wchar_t *Dir, size_t DirSizeInWords, wchar_t *Filename, 
										 size_t FilenameSizeInWords, wchar_t *Ext, size_t ExtSizeInWords)
{
	// Fake secure

	_wsplitpath(FullPath, Drive, Dir, Filename, Ext);

	return 0;
}


//
// Stdio Functions: scanf, wscanf
// very bad for optimization but I wrote them anyway
//

extern "C" int __cdecl scanf(const char *format, ...)
{
	va_list lst;
	const char *p = format;
	unsigned long BR;
	int fields = 0;

	// supported types

	int *SignedInteger;
	unsigned int *UnsignedInteger;
	char *Character;
	char *Str;

	//

	char *Buf = (char *) HeapAlloc(GetProcessHeap(), 0, (void *) 2048);

	va_start(lst, format);

	while (*p)
	{
		if (*p != 'd' && *p != 'u' && *p != 'x' && *p != 'X' && *p != 's' && *p != 'c') 
		{
			p++;
			continue;
		}

		// put the read console at the beginning to reduce instructions redudancy

		if (!ReadConsoleA(GetStdHandle(STD_INPUT_HANDLE), Buf, 2048, &BR, NULL))
			return -1;

		switch (*p)
		{

		case 'd':
			{
				SignedInteger = va_arg(lst, int *);

				*SignedInteger = (int) strtol(Buf, NULL, 10);
				
				break;
			}

		case 'u':
			{
				UnsignedInteger = va_arg(lst, unsigned int *);

				*UnsignedInteger = (unsigned int) strtoul(Buf, NULL, 10);

				break;
			}
			
		case 'x':
		case 'X':
			{
				UnsignedInteger = va_arg(lst, unsigned int *);

				*UnsignedInteger = (unsigned int) strtoul(Buf, NULL, 16);

				break;
			}

		case 'c':
			{
				Character = va_arg(lst, char *);

				*Character = (char) Buf[0];

				break;
			}

		case 's':
			{
				Str = va_arg(lst, char *);

				strncpy(Str, Buf, BR - 1);

				break;
			}

		/* Not supported
		
		case 'f':
			{
				break;
			}
			
			*/

		default:
			{
				// do nothing
				break;
			}
		}

		fields++;
		p++;
	}

	HeapFree(GetProcessHeap(), 0, Buf);

	return fields;
}

extern "C" int __cdecl wscanf(const wchar_t *format, ...)
{
	va_list lst;
	const wchar_t *p = format;
	unsigned long BR;
	int fields = 0;

	// supported types

	int *SignedInteger;
	unsigned int *UnsignedInteger;
	wchar_t *Character;
	wchar_t *Str;

	//

	wchar_t *Buf = (wchar_t *) HeapAlloc(GetProcessHeap(), 0, (void *) (2048 * sizeof (wchar_t)));

	va_start(lst, format);

	while (*p)
	{
		if (*p != 'd' && *p != 'u' && *p != 'x' && *p != 'X' && *p != 's' && *p != 'c') 
		{
			p++;
			continue;
		}

		// put the read console at the beginning to reduce instructions redudancy

		if (!ReadConsoleW(GetStdHandle(STD_INPUT_HANDLE), Buf, 2048, &BR, NULL))
			return -1;

		switch (*p)
		{

		case 'd':
			{
				SignedInteger = va_arg(lst, int *);

				*SignedInteger = (int) wcstol(Buf, NULL, 10);

				break;
			}

		case 'u':
			{
				UnsignedInteger = va_arg(lst, unsigned int *);

				*UnsignedInteger = (unsigned int) wcstoul(Buf, NULL, 10);

				break;
			}

		case 'x':
		case 'X':
			{
				UnsignedInteger = va_arg(lst, unsigned int *);

				*UnsignedInteger = (unsigned int) wcstoul(Buf, NULL, 16);

				break;
			}

		case 'c':
			{
				Character = va_arg(lst, wchar_t *);

				*Character = (wchar_t) Buf[0];

				break;
			}

		case 's':
			{
				Str = va_arg(lst, wchar_t *);

				wcsncpy(Str, Buf, BR - 1);

				break;
			}

			/* Not supported

		case 'f':
			{
				break;
			}

			*/

		default:
			{
				// do nothing
				break;
			}
		}

		fields++;
		p++;
	}

	HeapFree(GetProcessHeap(), 0, Buf);

	return fields;
}